import os, sys
import pytest
import itertools
import random

sys.path.insert(0, os.getcwd())

from unittest import TestCase

import networkx as nx
import numpy as np

import cosasi

from ..source_results import SourceResult, SingleSourceResult, MultiSourceResult


class Test_SourceResult(TestCase):
    def setUp(self):
        self.inference_method = "rumor centrality"
        self.scores = {1: 1, 2: 10, 3: 0.025}
        self.G = nx.fast_gnp_random_graph(50, 0.25)
        self.result = SourceResult(
            inference_method=self.inference_method,
            scores=self.scores,
            source_type="single-source",
            G=self.G,
        )
        return None

    def test_rank(self):
        ranked = self.result.rank()
        assert self.result.rank() == [2, 1, 3]

    def test_topn(self):
        with pytest.raises(ValueError):
            assert self.result.topn("BAD INPUT")
        assert self.result.topn(n=1) == [2]

    def test_bad_graph_input(self):
        with pytest.raises(ValueError):
            assert SourceResult(
                inference_method=self.inference_method,
                scores=self.scores,
                source_type="single-source",
                G="BAD INPUT",
            )


class Test_SingleSourceResult(TestCase):
    def setUp(self):
        self.inference_method = "rumor centrality"
        self.scores = {1: 1, 2: 10, 3: 0.025}
        self.G = nx.fast_gnp_random_graph(50, 0.25)
        self.result = SingleSourceResult(
            inference_method=self.inference_method,
            scores=self.scores,
            source_type="single-source",
            G=self.G,
        )
        return None

    def test_get_rank(self):
        assert self.result.get_rank(2) == 1
        assert self.result.get_rank(3) == 3

    def test_bad_graph_input(self):
        with pytest.raises(ValueError):
            assert SingleSourceResult(
                inference_method=self.inference_method,
                scores=self.scores,
                source_type="single-source",
                G="BAD INPUT",
            )


class Test_MultiSourceResult(TestCase):
    def setUp(self):
        self.inference_method = "fast multi-source jordan centrality"
        self.scores = {}
        self.G = nx.fast_gnp_random_graph(50, 0.25)
        for s in list(itertools.combinations(range(5), 2)):
            self.scores[s] = random.random()
        self.result = MultiSourceResult(
            inference_method=self.inference_method,
            scores=self.scores,
            source_type="multi-source",
            G=self.G,
        )
        return None

    def test_get_rank(self):
        max_key = max(self.scores, key=self.scores.get)
        assert self.result.get_rank(max_key) == 1

    def test_bad_graph_input(self):
        with pytest.raises(ValueError):
            assert MultiSourceResult(
                inference_method=self.inference_method,
                scores=self.scores,
                source_type="single-source",
                G="BAD INPUT",
            )



import os, sys

sys.path.insert(0, os.getcwd())

from unittest import TestCase

import pytest
import networkx as nx
import numpy as np

import cosasi


class TestJordan(TestCase):
    def setUp(self):
        self.G = nx.random_tree(n=500, seed=0)
        contagion = cosasi.StaticNetworkContagion(
            G=self.G, model="si", infection_rate=0.1, number_infected=1
        )
        contagion.forward(50)
        self.t = 25
        self.I = contagion.get_infected_subgraph(self.t)
        return None

    def test_jordan_centrality(self):
        result = cosasi.source_inference.single_source.jordan_centrality(self.I, self.G)
        # type check
        assert isinstance(
            result, cosasi.source_inference.source_results.SingleSourceResult
        )
        # soft eccentricity values should either be -inf or in [0, 1]
        noninf_vals = [i for i in result.data["scores"].values() if i != -np.inf]
        assert all(0 <= val <= 1 for val in noninf_vals)
        # confirm the set of nodes w/ highest score is the infection graph center
        center = list(nx.center(self.I))
        result_center = [
            i
            for i in result.data["scores"].keys()
            if result.data["scores"][i] == max(result.data["scores"].values())
        ]
        assert sorted(center) == sorted(result_center)



import os, sys

sys.path.insert(0, os.getcwd())

import pytest
from unittest import TestCase
import networkx as nx
import numpy as np

import cosasi


class Test_BenchmarkFromSimulation(TestCase):
    def setUp(self):
        self.number_infected_init = 3
        self.sim_steps = 100
        self.G = nx.fast_gnp_random_graph(100, 0.25)
        self.contagion = cosasi.StaticNetworkContagion(
            G=self.G,
            model="si",
            infection_rate=0.01,
            number_infected=self.number_infected_init,
        )
        self.contagion.forward(self.sim_steps)
        self.t = 15
        return None

    def test_inputs_contagion(self):
        with pytest.raises((AttributeError, ValueError)):
            cosasi.BenchmarkFromSimulation(
                contagion="BAD INPUT", information_type="single snapshot", t=self.t
            )
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion, information_type="single snapshot", t=self.t
        )
        benchmark.go()
        assert True

    def test_inputs_information_type(self):
        with pytest.raises(NotImplementedError):
            cosasi.BenchmarkFromSimulation(
                contagion=self.contagion, information_type="BAD INPUT", t=self.t
            )
        with pytest.raises(ValueError):
            cosasi.BenchmarkFromSimulation(
                contagion=self.contagion, information_type="observers", t=self.t
            )
        with pytest.raises(ValueError):
            cosasi.BenchmarkFromSimulation(
                contagion=self.contagion,
                information_type="observers",
                t=self.t,
                observers="BAD INPUT",
            )
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion,
            information_type="observers",
            t=self.t,
            observers=2,
        )
        # benchmark.go()
        assert True
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion,
            information_type="observers",
            t=self.t,
            observers=[0, 1],
        )
        # benchmark.go()
        assert True
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion, information_type="single snapshot", t=self.t
        )
        benchmark.go()
        assert True

    def test_inputs_t(self):
        with pytest.raises(ValueError):
            cosasi.BenchmarkFromSimulation(
                contagion=self.contagion,
                information_type="single snapshot",
                t="BAD INPUT",
            )
        with pytest.raises(ValueError):
            # invalid step
            cosasi.BenchmarkFromSimulation(
                contagion=self.contagion,
                information_type="single snapshot",
                t=self.sim_steps + 1,
            )
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion, information_type="single snapshot", t=self.t
        )
        benchmark.go()
        assert True

    def test_go_output(self):
        benchmark = cosasi.BenchmarkFromSimulation(
            contagion=self.contagion, information_type="single snapshot", t=self.t
        )
        results = benchmark.go()
        assert isinstance(results, dict)
        results_keys = results.keys()
        assert all(isinstance(k, str) for k in results_keys)
        assert all(isinstance(results[k], dict) for k in results_keys)
        assert all(
            isinstance(
                results[k]["source result"],
                (
                    cosasi.source_inference.source_results.SingleSourceResult,
                    cosasi.source_inference.source_results.MultiSourceResult,
                ),
            )
            for k in results_keys
        )


class Test_BenchmarkFromDetails(TestCase):
    def setUp(self):
        self.number_infected_init = 3
        self.sim_steps = 100
        self.G = nx.fast_gnp_random_graph(100, 0.25)
        self.contagion = cosasi.StaticNetworkContagion(
            G=self.G,
            model="si",
            infection_rate=0.01,
            number_infected=self.number_infected_init,
        )
        self.contagion.forward(self.sim_steps)
        self.t = 15
        self.I = self.contagion.get_infected_subgraph(step=self.t)
        self.true_source = self.contagion.get_source()
        return None

    def test_inputs_true_source(self):
        with pytest.raises(ValueError):
            benchmark = cosasi.BenchmarkFromDetails(
                true_source="BAD INPUT",
                G=self.G,
                I=self.I,
                t=self.t,
                number_sources=self.number_infected_init,
                information_type="single snapshot",
            )
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=self.number_infected_init,
            information_type="single snapshot",
        )
        benchmark.go()
        assert True

    def test_inputs_G(self):
        with pytest.raises(ValueError):
            benchmark = cosasi.BenchmarkFromDetails(
                true_source=self.true_source,
                G="BAD INPUT",
                I=self.I,
                t=self.t,
                number_sources=self.number_infected_init,
                information_type="single snapshot",
            )
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=self.number_infected_init,
            information_type="single snapshot",
        )
        benchmark.go()
        assert True

    def test_inputs_I(self):
        with pytest.raises(ValueError):
            benchmark = cosasi.BenchmarkFromDetails(
                true_source=self.true_source,
                G=self.G,
                I="BAD INPUT",
                t=self.t,
                number_sources=self.number_infected_init,
                information_type="single snapshot",
            )
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=self.number_infected_init,
            information_type="single snapshot",
        )
        benchmark.go()
        assert True

    def test_inputs_t(self):
        with pytest.raises(ValueError):
            benchmark = cosasi.BenchmarkFromDetails(
                true_source=self.true_source,
                G=self.G,
                I=self.I,
                t="BAD INPUT",
                number_sources=self.number_infected_init,
                information_type="single snapshot",
            )
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=self.number_infected_init,
            information_type="single snapshot",
        )
        benchmark.go()
        assert True

    def test_inputs_number_sources(self):
        with pytest.raises(TypeError):
            benchmark = cosasi.BenchmarkFromDetails(
                true_source=self.true_source,
                G=self.G,
                I=self.I,
                t=self.t,
                number_sources="BAD INPUT",
                information_type="single snapshot",
            )
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=self.number_infected_init,
            information_type="single snapshot",
        )
        benchmark.go()
        assert True

    def test_get_namespaces(self):
        # single source should not have any multisource algorithms
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=1,
            information_type="single snapshot",
        )
        namespaces = [n.__name__ for n in benchmark.get_namespaces()]
        for n in namespaces:
            if "fast_multisource" in n:
                assert False
        assert True
        # multi-source should have multisource algorithms
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=3,
            information_type="single snapshot",
        )
        namespaces = [n.__name__ for n in benchmark.get_namespaces()]
        temp = False
        for n in namespaces:
            if "fast_multisource" in n:
                temp = True
                break
        assert temp

    def test_go_output(self):
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=self.true_source,
            G=self.G,
            I=self.I,
            t=self.t,
            number_sources=None,
            information_type="single snapshot",
        )
        results = benchmark.go()
        assert isinstance(results, dict)
        results_keys = results.keys()
        assert all(isinstance(k, str) for k in results_keys)
        assert all(isinstance(results[k], dict) for k in results_keys)
        assert all(
            isinstance(
                results[k]["source result"],
                (
                    cosasi.source_inference.source_results.SingleSourceResult,
                    cosasi.source_inference.source_results.MultiSourceResult,
                ),
            )
            for k in results_keys
        )

    def test_go_with_observers(self):
        G = nx.fast_gnp_random_graph(100, 0.25)
        contagion = cosasi.StaticNetworkContagion(
            G=G, model="si", infection_rate=0.01, number_infected=1
        )
        contagion.forward(100)
        I = contagion.get_infected_subgraph(step=25)
        observers = contagion.get_observers(10)
        true_source = contagion.get_source()
        benchmark = cosasi.BenchmarkFromDetails(
            true_source=true_source,
            G=G,
            I=I,
            t=15,
            number_sources=len(true_source),
            information_type="observers",
            observer_dict=observers,
        )
        results = benchmark.go()
        assert "single-source earliest infection first" in results.keys()
        assert "single-source lisn" not in results.keys()



import os, sys

sys.path.insert(0, os.getcwd())

from unittest import TestCase

import pytest
import networkx as nx
import numpy as np
import random
import math

import cosasi


class TestNETSLEUTH(TestCase):
    def setUp(self):
        self.G = nx.random_tree(n=500, seed=0)
        contagion = cosasi.StaticNetworkContagion(
            G=self.G, model="si", infection_rate=0.1, number_infected=1
        )
        contagion.forward(50)
        self.t = 25
        self.I = contagion.get_infected_subgraph(self.t)
        return None

    def test_netsleuth(self):
        result = cosasi.source_inference.single_source.netsleuth(self.I, self.G)
        assert isinstance(
            result, cosasi.source_inference.source_results.SingleSourceResult
        )
        noninf_vals = [i for i in result.data["scores"].values() if i != -np.inf]
        assert len(noninf_vals) == len(self.I)



import os, sys

sys.path.insert(0, os.getcwd())

from unittest import TestCase

import pytest
import networkx as nx

import cosasi


class TestJordan(TestCase):
    def setUp(self):
        self.G = nx.complete_graph(n=100)
        contagion = cosasi.StaticNetworkContagion(
            G=self.G, model="si", infection_rate=0.1, number_infected=1
        )
        contagion.forward(50)
        self.t = 25
        self.I = contagion.get_infected_subgraph(self.t)
        return None

    def test_fast_multisource_jordan_centrality(self):
        result = cosasi.source_inference.multiple_source.fast_multisource_jordan_centrality(
            self.I, self.G, 3
        )
        assert isinstance(
            result, cosasi.source_inference.source_results.MultiSourceResult
        )
        top5 = result.topn(5)
        assert [len(i) == 3 for i in top5]
        result = cosasi.source_inference.multiple_source.fast_multisource_jordan_centrality(
            self.I, self.G
        )
        l = None
        for k in result.data["scores"].keys():
            if not l:
                l = len(k)
            else:
                assert len(k) == l



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="cosasi",
    version="0.0.1",
    author="Lucas H. McCabe",
    author_email="lmccabe@lmi.org",
    description="COntagion Simulation And Source Inference",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lmiconsulting/cosasi",
    packages=find_packages(),
    package_data={"cosasi": ["source_inference/algorithm_details.json"]},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



import os, sys

sys.path.insert(0, os.getcwd())

import pytest
import networkx as nx
import numpy as np
import random

from cosasi import utils


def test_list_product():
    l = [1]
    assert utils.list_product(l) == 1
    l += [2]
    assert utils.list_product(l) == 2
    l += [-3]
    assert utils.list_product(l) == -6
    l += [0]
    assert utils.list_product(l) == 0
    return None


def test_longest_list():
    n = 10
    l = []
    for i in range(n):
        l.append(list(range(i)))
    longest = utils.longest_list(l)
    assert longest == list(range(n - 1))


def test_longest_list_len():
    n = 10
    l = []
    for i in range(n):
        l.append(list(range(i)))
    assert utils.longest_list_len(l) == n - 1


def test_soft_eccentricity():
    G = nx.complete_graph(4)
    assert utils.soft_eccentricity(G, 1) < np.inf
    H = nx.disjoint_union(G, G)
    assert utils.soft_eccentricity(H, 1) == np.inf
    G = nx.complete_graph(1)
    assert utils.soft_eccentricity(G, 0) == 1


def test_attack_degree():
    G = nx.complete_graph(4)
    infected = [3]
    for i in range(3):
        assert utils.attack_degree(infected, G, i) == 1


def attack_degree_partition():
    G = nx.gnp_random_graph(50, 0.2)
    node_set = [1, 3, 4]
    infected = [1, 16, 17, 19, 24, 34, 36, 41, 43, 49]
    partition = utils.attack_degree_partition(node_set, infected, G)
    vals = []
    for v in partition.values():
        vals += v
    assert sorted(vals) == sorted(node_set)
    assert max(partition) <= max(G.degree())[1]
